/*
 * Arduino code to control a model locomotive using a rotary encoder.
 * 
 * Watch the video:-  https://youtu.be/bh8KguloqlQ
 * 
 * Made by Tech Build:- https://www.youtube.com/channel/UCNy7DyfhSD9jsQEgNwETp9g?sub_confirmation=1
 * 
 * Feel free to modify the code to adapt it to your application.
 */
#include <Adafruit_MotorShield.h>

Adafruit_MotorShield AFMS = Adafruit_MotorShield();

Adafruit_DCMotor *motorA = AFMS.getMotor(4);


#define inputA 6
#define inputB 7

int AaState;
int AaLastState;
int sA = 0;
int spA = 0;
long pulseA;

void motorA_run(int r) {

  if (r >= 0 ) {
    motorA->setSpeed(r);
    motorA->run(FORWARD);
  }
  if (r < 0 ) {
    motorA->setSpeed(-r);
    motorA->run(BACKWARD);
  }
  if (r == 0) {
    motorA->run(RELEASE);
  }
}

void motorA_halt() {

  while (spA != 0) {

    Serial.println(sA);

    AaState = digitalRead(inputA);

    if (AaState != AaLastState) break;

    sA = map(spA, -512, 512, -255, 255);

    motorA_run(sA);

    if (spA > 0)  spA--;

    if (spA < 0)  spA++;

    delay(30);
  }
}

void setup() {
  pinMode (inputA, INPUT);
  pinMode (inputB, INPUT);

  AFMS.begin(950);

  Serial.begin (9600);
  // Reads the initial state of the inputA
  AaLastState = digitalRead(inputA);

}

void loop() {
  AaState = digitalRead(inputA); // Reads the "current" state of the inputA
  // If the previous and the current state of the inputA are different, that means a Pulse has occured
  if (AaState != AaLastState) {

    if (digitalRead(inputB) != AaState && spA < 512) {
      pulseA = pulseIn(inputA, HIGH);//Reads the pulse of the digital input pin.
                                     //The lower the value of 'pulseA', the faster is the encoder rotated.

      if (pulseA < 0) pulseA = 6000;

      if (pulseA > 4000 && spA < 512) {//If the encoder is rotated slowly in one direction(say clockwise).
        spA++;//Increase the speed of the locomotive in the forward direction.
      } else if (pulseA < 3500 && pulseA > 0) {//If the encoder is rotated quickly enough.
        motorA_halt();//Bring the locomotive to a halt.             
      }

    } else {
      pulseA = pulseIn(inputA, HIGH);

      if (pulseA < 0) pulseA = 6000;

      if (pulseA > 4000 && spA > -512) {//If the encoder is rotated slowly in the other direction(say anticlockwise).
        spA--;//Decrease the speed of the locomotive, if it is at rest, increase the speed in reverse direction
              //as the value of 'spA' will start to become -ve.      
      } else if (pulseA < 3500 && pulseA > 0) {//If the encoder is rotated quickly enough.
        motorA_halt();//Bring the locomotive to a halt.
      }

    }

    sA = map(spA, -512, 512, -255, 255);

    /*
    Serial.print(pulseA);
    Serial.print("   ");
    Serial.println(sA);
    */
    motorA_run(sA);




    // If the inputB state is different to the inputA state, that means the encoder is rotating clockwise


  }
  AaLastState = AaState; // Updates the previous state of the inputA with the current state

}
